import re

def toUUID(query:str):
    """
    Convert a standard name (author/topic) to a formatted UUID.

    Args:
        query (str): The query to convert.

    Returns:
        str: The formatted version (UUID).

    Raises:
        ValueError: Not specifing a query triggers this error.

    Example:
        Converting the name H. Jackson Brown, Jr. to an UUID.

        >>> print(toUUID('H. Jackson Brown, Jr.'))
        h-jackson-brown-jr
    """
    if(not query):
        raise ValueError('Specify a query bud.')
    return re.sub('(^-|-$)','',re.sub('--','-',re.sub('[^a-z]','-',query.lower())))



from requests import get
from bs4 import BeautifulSoup
import re



def most_viewed():
    """

    Retrieve the most viewed topics.

    Returns:
        list<``Topic``>: The topics.

    Raises:
        Error: Resulting from requesting the website

    """
    try:
        endpoint = f'https://www.brainyquote.com/topics'
        body = get(endpoint).text
        parsed_body = BeautifulSoup(body, 'html.parser')
        data = []
        for a in parsed_body.select_one('body > div.container.bqTopLevel > div.row.bq_left').find_all('a'):
            data.append(Topic(re.sub('(/topics/|-quotes)','',a.get('href'))))
        return data
    except Exception as e:
        raise e

class Topic(object):

    def __init__(self, UUID:str):
        """
        Create a topic object wich allows access to quotes related to this topic.

        Args:
            UUID (str): A special formatted version of the topic name , all lowercase and special chars removed. Check the ``misc.toUUID()`` to convert a name to an UUID.
        """
        self.UUID = UUID

    def quotes(self, page:int=1):
        from quotes import Quote
        """
        Retrieve the quotes related to this topic.

        Returns:
            list<``quotes.Quote``>: The quotes related to this topic.

        Raises:
            ValueError: That UUID is probably wrong bud.
            TypeError: Invalid type or format of the page parameter.

        """
        try:
            if(not type(page) == int or page < 1):
                raise TypeError('Invalid type or format of the page parameter.')
            endpoint = f'https://www.brainyquote.com/topics/{self.UUID}-quotes_{page}'
            body = get(endpoint).text
            parsed_body  = BeautifulSoup(body, 'html.parser')
            data = {
                'current_page': page,
                'total_pages': int(parsed_body.select_one('body > div.infScrollFooter > div.bq_s.hideInfScroll.bq_pageNumbersCont > nav > ul').find_all('li')[len(parsed_body.select_one('body > div.infScrollFooter > div.bq_s.hideInfScroll.bq_pageNumbersCont > nav > ul').find_all('li')) - 2].get_text()),
                'elements': []
            }
            for quote in parsed_body.select_one('#quotesList').find_all('div'):
                try:
                    data['elements'].append(Quote(re.sub('/quotes/','',quote.select_one('div > div:nth-child(1) > div > a').get('href'))))
                except Exception as e:
                    continue
            data['on_page'] = len(data['elements'])
            return data
        except Exception as e:
            raise ValueError('That UUID is probably wrong bud.')



from requests import get
from bs4 import BeautifulSoup
import re
from datetime import datetime as dt


def random():
    """
    Generate a random quote.

    Returns:
        Quote: The random qoute.

    """
    from topics import most_viewed
    from random import choice
    random_topic = choice(most_viewed())
    random_quote = choice(random_topic.quotes()['elements'])
    return random_quote
def top_30():
    """
    Top 30 quotes of all time, based on user selections.

    Returns:
        dict<`int`:``Quote``>: A dict holding position and the quote as an instance of a ``Quote`` class.

    Raises:
        Error: Resulting from requesting the website
    """
    try:
        endpoint = 'https://www.brainyquote.com/top_100_quotes'
        body = get(endpoint).text
        parsed_body = BeautifulSoup(body, 'html.parser')
        data = {}
        i = 1
        for position in parsed_body.select('.listPositionNum'):
            if(i == 31):
                break
            data[i] = Quote(re.sub('/quotes/','',position.parent.find('a').get('href')))
            i = i+1

        for light_position in parsed_body.select('.listPositionNumLight'):
            if(i == 31):
                break
            data[i] = Quote(re.sub('/quotes/','',light_position.parent.find('a').get('href')))
            i = i+1
        return data
    except Exception as e:
        raise e


def todays_pick():
    """
    Todays quote, updates every 24 hours.

    Returns:
        dict<`str`:```Quote``|`str`>: A dict holding the date and the quote as an instance of a ``Quote`` class.

    Raises:
        Error: Resulting from requesting the website

    Example:

        The pick of Moday, June 08 th, 2020
        >>> print(quotes.todays_pick()['quote'].snippet()['body'])
        The backbone of surprise is fusing speed with secrecy.
    """
    try:
        endpoint = 'https://www.brainyquote.com/quote_of_the_day'
        body = get(endpoint).text
        parsed_body = BeautifulSoup(body, 'html.parser')
        return {
            'date': dt.now(),
            'quote': Quote(re.sub('/quotes/','',parsed_body.select_one('body > div.qotd_days.m_panel > div:nth-child(4) > div > div > div > div:nth-child(2) > div > a').get('href')))
        }
    except Exception as e:
        raise e
class Quote():

    def __init__(self, UUID:str):
        """
        Create a quote object wich allows access to every piece of data available on the quote including the body (both text and image if available), author and related topics/authors/quotes.

        Args:
            UUID (str): A special formatted version of the name including a special id to suit the scraping system.
        """
        self.UUID = UUID

    def snippet(self):
        """
            Retrieve the quote's body and author.

            Returns:
                dict<`str`>: The dict that holds the info.

            Raises:
                ValueError: That UUID is probably wrong bud.

            Note:
                Includes instances of the ``Author`` class
        """
        from authors import Author
        try:
            endpoint = f'https://www.brainyquote.com/quotes/{self.UUID}'
            body = get(endpoint).text
            parsed_body = BeautifulSoup(body, 'html.parser')
            return {
                'body': parsed_body.select_one('#quotePageTopHolder > div > div > div.col-sm-7.col-md-8 > div > div > div > div > div.quoteContent > div > p').get_text(),
                'author': Author(re.sub('(/authors/|-quotes)','',parsed_body.select_one('#quotePageTopHolder > div > div > div.col-sm-7.col-md-8 > div > div > div > div > div.quoteContent > div > p.bq_fq_a > a').get('href')))
            }
        except Exception as e:
            raise ValueError('It seems like that UUID is invalid bud.')
    def image_content(self, background_id:int=1):
        """
            Retrieve the quote's image version if available, the background is customizable.

            Args:
                background_id (int, optional): The background id, defaults to 1.

            Returns:
                str: The image url.
                None: The image version is not available for this quote.

            Raises:
                ValueError: That UUID is probably wrong bud.

        """
        try:
            endpoint = f'https://www.brainyquote.com/quotes/{self.UUID}?img={background_id}'
            body = get(endpoint).text
            parsed_body = BeautifulSoup(body, 'html.parser')
            return  'https://www.brainyquote.com{}'.format(parsed_body.select_one('div.quoteContent > div > a > img').get('data-img-url')) if parsed_body.select_one('div.quoteContent > div > a > img') else None
        except Exception as e:
            raise ValueError('It seems like that UUID / background id is invalid  bud.')

    def related_quotes(self):
        """
        Retrieve related quotes as instances of the ``Quote`` class.

        Returns:
            list<``Quote``>: The list of the related quotes.

        Raises:
            ValueError: It seems like that UUID is invalid bud.
        """
        from quotes import Quote
        try:
            endpoint = f'https://www.brainyquote.com/quotes/{self.UUID}'
            body = get(endpoint).text
            parsed_body = BeautifulSoup(body, 'html.parser')
            data = []
            for quote in parsed_body.select_one('#quotesList').find_all('div'):
                try:
                    data.append(Quote(re.sub('/quotes/','',quote.select_one('div > div:nth-child(1) > div > a').get('href'))))
                except Exception as e:
                    continue
            return data
        except Exception as e:
            raise ValueError('It seems like that UUID is invalid bud.')



import setuptools

with open('README.md', 'r') as readme:
  long_description = readme.read()
  
setuptools.setup(
      name='advanced_quotes',
      version='1.0',
      description='Your all-access pass to the world\'s largest quotations database. Random , daily quotes , over 40 thousand authors, various topics. Inside your comfy python development environment!',
      url='https://github.com/Mahdios/advanced_quotes',
      author='Mahdi Djaber',
      author_email='mahdios.dj@gmail.com',
      license='GPL-3.0',
      packages=setuptools.find_packages(),
      zip_safe=False,
      long_description_content_type="text/markdown",
      long_description=long_description,
      install_requires=['bs4','requests']
)



from requests import get
from bs4 import BeautifulSoup
import re

def index(letter:str='a', page:int=1):
    """

    Retrieve all the authors whom name starts with a given letter

    Args:
        letter (str, optional): The letter to filter by (case unsensitive), defaults to 'a'.
        page: (int, optional): Due to high number of authors, the authors are split in pages (typically 200 per page). Make sure the letter you're giving contains enough authors. Defaults to 1.

    Returns:
        list<``Author``>: The authors related to this letter

    Raises:
        TypeError: In case you gave a param of wrong type

    Examples:
        Viewing the first a author's snippet

        >>> print(all()['authors'][0].snippet())
        {'nationality': 'American', 'occupation': 'Musician', 'dates': {'birth': {'date': 'December 6', 'year': '1995'}, 'death': None}, 'related_authors': [<__main__.Athor object at 0x033898D0>, <__main__.Author object at 0x03389890>, <__main__.Author object at 0x03389D10>, <__main__.Author object at 0x03389D50>, <__main__.Author object at 0x03389D70>, <__main__.Author object at 0x03389DB0>, <__main__.Author object at 0x03389DD0>, <__main__.Author object at 0x03389DF0>]}

        Viewing all of 'a' letter authors

        >>> print(all()['authors'])
        {'authors': [<__main__.Author object at 0x03242CB0>...200], 'total_pages': 12, 'on_page': 200}
    """
    try:
        if(not type(page) == int):
            raise TypeError('The page must be a string')
        authors = f'https://www.brainyquote.com/authors/{letter.lower()}{str(page)}'
        body = get(authors).text
        parsed_body = BeautifulSoup(body, 'html.parser')
        data = {
            'authors': [],
            'total_pages': int(parsed_body.select_one('body > div.container.bqTopLevel > div > div.col-sm-8.col-md-8 > div:nth-child(4) > div > div > div > div > div > div > nav > ul').find_all('li')[len(parsed_body.select_one('body > div.container.bqTopLevel > div > div.col-sm-8.col-md-8 > div:nth-child(4) > div > div > div > div > div > div > nav > ul').find_all('li')) - 2].get_text()),
        }
        for tr in parsed_body.find('tbody').find_all('tr'):
            data['authors'].append(Author(tr.select_one('td:nth-child(1) > a').get('href').strip()))
        data['on_page'] = len(data['authors'])
        return data
    except Exception as e:
        raise e

def most_viewed(letter:str='a'):
    """

    Retrieve the most viewed authors of a given letter

    Args:
        letter (str, optional): The letter to filter by (case unsensitive), defaults to 'a'.

    Returns:
        list<``Author``>: The authors.

    Raises:
        TypeError: In case you gave a param of wrong type

    """
    try:
        endpoint = f'https://www.brainyquote.com/authors/{letter.lower()}'
        body = get(endpoint).text
        parsed_body = BeautifulSoup(body, 'html.parser')
        data = []
        for a in parsed_body.select('a.block-sm-az'):
            data.append(Author(re.sub('/authors/','',a.get('href'))))
        return data
    except Exception as e:
        raise e

class Author():
    def __init__(self, UUID:str):
        """
        Create an author object wich allows access to every piece of data available on the author including the name, a set of his quotes, biography and full on social info.

        Args:
            UUID (str): A special formatted version of the author's name , all lowercase and special chars removed. Check the ``misc.toUUID()`` to convert a name to an UUID
        """
        self.UUID = UUID

    def snippet(self):
        """
        Basic info about the author such as nationality, occupation, birth/death and related authors

        Returns:
            dict<`str`>: The dict that holds the info.

        Raises:
            ValueError: That UUID is probably wrong bud.

        Note:
            Includes instances of the ``Author`` class
        """
        try:
            endpoint = f'https://www.brainyquote.com/authors/{self.UUID}'
            body = get(endpoint).text
            parsed_body  = BeautifulSoup(body, 'html.parser')
            data = {
                'name': re.sub('Quotes', '',parsed_body.select_one('#bqNavBarCtrl > div > nav > div.navbar-header > h1').get_text()),
                'nationality': parsed_body.select_one('body > div.subnav-below-p > a:nth-child(1)').get_text(),
                'occupation': parsed_body.select_one('body > div.subnav-below-p > a:nth-child(2)').get_text(),
                'related_authors': []
            }
            for author in parsed_body.select_one('body > div.infScrollFooter > div.container-fluid > div > div.col-sm-7.col-md-8.block-style > div.bq_s > div').find_all('div'):
                data['related_authors'].append(Author(re.sub('Quotes', '', author.select_one('.link-name').get_text().strip())))
            return data
        except Exception as e:
            raise ValueError('That UUID is probably wrong bud.')

    def biography(self):
        """
        Retrieve the author's biography if available.

        Returns:
            str: The biography.

        Raises:
            ValueError: That UUID is wrong or the biography is not available.
        """
        try:
            endpoint = f'https://www.brainyquote.com/quotes/biography/{self.UUID}'
            body = get(endpoint).text
            parsed_body  = BeautifulSoup(body, 'html.parser')
            return parsed_body.select_one('div.bio-content').get_text().strip()
        except Exception as e:
            raise ValueError('That UUID is wrong or the biography is not available.')


    def quotes(self, page:int=1):
        """
        Retrieve the author's set of quotes, instances of the ``Quote`` class

        Args:
            page: (int, optional): Due to high number of quotes sometimes, they are split in pages. Make sure the author you've selected has enough pages. Defaults to 1.

        Returns:
            list<``quotes.Quote``>: The set of quotes.

        Raises:
            ValueError: That UUID is probably wrong bud.
            TypeError: Invalid type or format of the page parameter.

        Example:
            Retrieve the first page of abraham lincoln quotes

            >>> print(Author('abraham-lincoln').quotes())
            {'current_page': 1, 'total_pages': 9, 'elements': [<quotes.Quote object at 0x031A6C10>...165], 'on_page': 165}
        """
        from quotes import Quote
        try:
            if(not type(page) == int or page < 1):
                raise TypeError('Invalid type or format of the page parameter.')
            endpoint = f'https://www.brainyquote.com/authors/{self.UUID}-quotes_{page}'
            body = get(endpoint).text
            parsed_body  = BeautifulSoup(body, 'html.parser')
            data = {
                'current_page': page,
                'total_pages': int(parsed_body.select_one('body > div.infScrollFooter > div.bq_s.hideInfScroll.bq_pageNumbersCont > nav > ul').find_all('li')[len(parsed_body.select_one('body > div.infScrollFooter > div.bq_s.hideInfScroll.bq_pageNumbersCont > nav > ul').find_all('li')) - 2].get_text()),
                'elements': []
            }
            for quote in parsed_body.select_one('#quotesList').find_all('div'):
                try:
                    data['elements'].append(Quote(re.sub('/quotes/','',quote.select_one('div > div:nth-child(1) > div > a').get('href'))))
                except Exception as e:
                    continue
            data['on_page'] = len(data['elements'])
            return data
        except Exception as e:
            raise ValueError('That UUID is probably wrong bud.')
